@@ -23,11 +23,19 @@
 #include <private/qqmlsourcecoordinate_p.h>
 #include <private/qqmlsignalnames_p.h>
 
-#include <QScopedValueRollback>
+#include <QtCore/qloggingcategory.h>
+#include <QtCore/qscopedvaluerollback.h>
+
+#if QT_CONFIG(regularexpression)
+#include <QtCore/qregularexpression.h>
+#endif
+
 #include <vector>
 
 QT_BEGIN_NAMESPACE
 
+Q_DECLARE_LOGGING_CATEGORY(invalidOverride);
+
 inline QQmlError qQmlCompileError(const QV4::CompiledData::Location &location,
                                                   const QString &description)
 {
@@ -67,8 +75,96 @@ struct QQmlPropertyCacheCreatorBase
 public:
     static QAtomicInt Q_AUTOTEST_EXPORT classIndexCounter;
 
-    static QMetaType metaTypeForPropertyType(QV4::CompiledData::CommonType type);
-    static QMetaType listTypeForPropertyType(QV4::CompiledData::CommonType type);
+    static QMetaType metaTypeForPropertyType(QV4::CompiledData::CommonType type)
+    {
+        switch (type) {
+        case QV4::CompiledData::CommonType::Void:     return QMetaType();
+        case QV4::CompiledData::CommonType::Var:      return QMetaType::fromType<QVariant>();
+        case QV4::CompiledData::CommonType::Int:      return QMetaType::fromType<int>();
+        case QV4::CompiledData::CommonType::Bool:     return QMetaType::fromType<bool>();
+        case QV4::CompiledData::CommonType::Real:     return QMetaType::fromType<qreal>();
+        case QV4::CompiledData::CommonType::String:   return QMetaType::fromType<QString>();
+        case QV4::CompiledData::CommonType::Url:      return QMetaType::fromType<QUrl>();
+        case QV4::CompiledData::CommonType::Time:     return QMetaType::fromType<QTime>();
+        case QV4::CompiledData::CommonType::Date:     return QMetaType::fromType<QDate>();
+        case QV4::CompiledData::CommonType::DateTime: return QMetaType::fromType<QDateTime>();
+#if QT_CONFIG(regularexpression)
+        case QV4::CompiledData::CommonType::RegExp:   return QMetaType::fromType<QRegularExpression>();
+#else
+        case QV4::CompiledData::CommonType::RegExp:   return QMetaType();
+#endif
+        case QV4::CompiledData::CommonType::Rect:     return QMetaType::fromType<QRectF>();
+        case QV4::CompiledData::CommonType::Point:    return QMetaType::fromType<QPointF>();
+        case QV4::CompiledData::CommonType::Size:     return QMetaType::fromType<QSizeF>();
+        case QV4::CompiledData::CommonType::Invalid:  break;
+        };
+        return QMetaType {};
+    }
+
+    static QMetaType listTypeForPropertyType(QV4::CompiledData::CommonType type)
+    {
+        switch (type) {
+        case QV4::CompiledData::CommonType::Void:     return QMetaType();
+        case QV4::CompiledData::CommonType::Var:      return QMetaType::fromType<QList<QVariant>>();
+        case QV4::CompiledData::CommonType::Int:      return QMetaType::fromType<QList<int>>();
+        case QV4::CompiledData::CommonType::Bool:     return QMetaType::fromType<QList<bool>>();
+        case QV4::CompiledData::CommonType::Real:     return QMetaType::fromType<QList<qreal>>();
+        case QV4::CompiledData::CommonType::String:   return QMetaType::fromType<QList<QString>>();
+        case QV4::CompiledData::CommonType::Url:      return QMetaType::fromType<QList<QUrl>>();
+        case QV4::CompiledData::CommonType::Time:     return QMetaType::fromType<QList<QTime>>();
+        case QV4::CompiledData::CommonType::Date:     return QMetaType::fromType<QList<QDate>>();
+        case QV4::CompiledData::CommonType::DateTime: return QMetaType::fromType<QList<QDateTime>>();
+#if QT_CONFIG(regularexpression)
+        case QV4::CompiledData::CommonType::RegExp:   return QMetaType::fromType<QList<QRegularExpression>>();
+#else
+        case QV4::CompiledData::CommonType::RegExp:   return QMetaType();
+#endif
+        case QV4::CompiledData::CommonType::Rect:     return QMetaType::fromType<QList<QRectF>>();
+        case QV4::CompiledData::CommonType::Point:    return QMetaType::fromType<QList<QPointF>>();
+        case QV4::CompiledData::CommonType::Size:     return QMetaType::fromType<QList<QSizeF>>();
+        case QV4::CompiledData::CommonType::Invalid:  break;
+        };
+        return QMetaType {};
+    }
+
+    static QV4::CompiledData::CommonType propertyTypeForMetaType(QMetaType metaType) {
+        if (!metaType.isValid())
+            return QV4::CompiledData::CommonType::Void;
+
+        switch (metaType.id()) {
+        case QMetaType::QVariant:
+            return QV4::CompiledData::CommonType::Var;
+        case QMetaType::Int:
+            return QV4::CompiledData::CommonType::Int;
+        case QMetaType::Bool:
+            return QV4::CompiledData::CommonType::Bool;
+        case QMetaType::QReal:
+            return QV4::CompiledData::CommonType::Real;
+        case QMetaType::QString:
+            return QV4::CompiledData::CommonType::String;
+        case QMetaType::QUrl:
+            return QV4::CompiledData::CommonType::Url;
+        case QMetaType::QTime:
+            return QV4::CompiledData::CommonType::Time;
+        case QMetaType::QDate:
+            return QV4::CompiledData::CommonType::Date;
+        case QMetaType::QDateTime:
+            return QV4::CompiledData::CommonType::DateTime;
+#if QT_CONFIG(regularexpression)
+        case QMetaType::QRegularExpression:
+            return QV4::CompiledData::CommonType::RegExp;
+#endif
+        case QMetaType::QRectF:
+            return QV4::CompiledData::CommonType::Rect;
+        case QMetaType::QPointF:
+            return QV4::CompiledData::CommonType::Point;
+        case QMetaType::QSizeF:
+            return QV4::CompiledData::CommonType::Size;
+        default:
+            break;
+        }
+        return QV4::CompiledData::CommonType::Invalid;
+    }
 
     static bool canCreateClassNameTypeByUrl(const QUrl &url);
     static QByteArray createClassNameTypeByUrl(const QUrl &url);
@@ -235,8 +331,10 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::buildMetaObjectRecur
     };
 
     const CompiledObject *obj = objectContainer->objectAt(objectIndex);
-    bool needVMEMetaObject = isVMERequired == VMEMetaObjectIsRequired::Always || obj->propertyCount() != 0 || obj->aliasCount() != 0
-            || obj->signalCount() != 0 || obj->functionCount() != 0 || obj->enumCount() != 0
+    bool needVMEMetaObject = isVMERequired == VMEMetaObjectIsRequired::Always
+            || obj->propertyCount() != 0 || obj->aliasCount() != 0 || obj->signalCount() != 0
+            || obj->functionCount() != 0 || obj->enumCount() != 0
+            || obj->inlineComponentCount() != 0
             || ((obj->hasFlag(QV4::CompiledData::Object::IsComponent)
                  || (objectIndex == 0 && isAddressable(objectContainer->url())))
                 && !objectContainer->resolvedType(obj->inheritedTypeNameIndex)->isFullyDynamicType());
@@ -364,8 +462,9 @@ inline QQmlPropertyCache::ConstPtr QQmlPropertyCacheCreator<ObjectContainer>::pr
             Q_ASSERT(typeRef);
             QQmlType qmltype = typeRef->type();
             if (!qmltype.isValid()) {
-                imports->resolveType(stringAt(binding->propertyNameIndex),
-                                     &qmltype, nullptr, nullptr, nullptr);
+                imports->resolveType(
+                        QQmlTypeLoader::get(enginePrivate), stringAt(binding->propertyNameIndex),
+                        &qmltype, nullptr, nullptr);
             }
 
             const QMetaObject *attachedMo = qmltype.attachedPropertiesType(enginePrivate);
@@ -564,7 +663,7 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
                 return message;
             case AllowOverride::Yes:
                 message.setUrl(objectContainer->url());
-                enginePrivate->warning(message);
+                qCWarning(invalidOverride).noquote() << message.toString();
                 *it = AllowOverride::No; // No further overriding allowed.
                 break;
             }
@@ -594,7 +693,7 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
                 return message;
             case AllowOverride::Yes:
                 message.setUrl(objectContainer->url());
-                enginePrivate->warning(message);
+                qCWarning(invalidOverride).noquote() << message.toString();
                 break;
             }
         }
@@ -649,8 +748,9 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
             QQmlType qmltype;
             bool selfReference = false;
             if (!imports->resolveType(
-                    stringAt(p->commonTypeOrTypeNameIndex()), &qmltype, nullptr, nullptr,
-                    nullptr, QQmlType::AnyRegistrationType, &selfReference)) {
+                        QQmlTypeLoader::get(enginePrivate),
+                        stringAt(p->commonTypeOrTypeNameIndex()), &qmltype, nullptr, nullptr,
+                        nullptr, QQmlType::AnyRegistrationType, &selfReference)) {
                 return qQmlCompileError(p->location, QQmlPropertyCacheCreatorBase::tr("Invalid property type"));
             }
 
@@ -728,8 +828,9 @@ inline QMetaType QQmlPropertyCacheCreator<ObjectContainer>::metaTypeForParameter
         *customTypeName = typeName;
     QQmlType qmltype;
     bool selfReference = false;
-    if (!imports->resolveType(typeName, &qmltype, nullptr, nullptr, nullptr,
-                              QQmlType::AnyRegistrationType, &selfReference))
+    if (!imports->resolveType(
+                &enginePrivate->typeLoader, typeName, &qmltype, nullptr, nullptr, nullptr,
+                QQmlType::AnyRegistrationType, &selfReference))
         return QMetaType();
 
     if (!qmltype.isComposite()) {
@@ -856,7 +957,7 @@ inline QQmlError QQmlPropertyCacheAliasCreator<ObjectContainer>::propertyDataFor
                 Q_ASSERT(type->isValid());
             }
         } else {
-            *type = typeRef->compilationUnit()->qmlType.typeId();
+            *type = typeRef->compilationUnit()->metaType();
         }
 
         *version = typeRef->version();
