@@ -19,6 +19,7 @@
 #include <private/qqmldebugserviceinterfaces_p.h>
 
 #include <QtQml/qqmlengine.h>
+#include <QtQml/qqmlcomponent.h>
 #include <private/qqmlengine_p.h>
 #include <QtCore/qbasictimer.h>
 #include <QtGui/QOffscreenSurface>
@@ -276,9 +277,7 @@ void QQuickWidgetPrivate::handleWindowChange()
     QObject::connect(renderControl, SIGNAL(renderRequested()), q, SLOT(triggerUpdate()));
     QObject::connect(renderControl, SIGNAL(sceneChanged()), q, SLOT(triggerUpdate()));
 
-    if (!source.isEmpty())
-        execute();
-    else if (QQuickItem *sgItem = qobject_cast<QQuickItem *>(root))
+    if (QQuickItem *sgItem = qobject_cast<QQuickItem *>(root))
         sgItem->setParentItem(offscreenWindow->contentItem());
 }
 
@@ -305,6 +304,20 @@ QQuickWidgetPrivate::QQuickWidgetPrivate()
 {
 }
 
+void QQuickWidgetPrivate::executeHelper()
+{
+    ensureEngine();
+
+    if (root) {
+        delete root;
+        root = nullptr;
+    }
+    if (component) {
+        delete component;
+        component = nullptr;
+    }
+}
+
 void QQuickWidgetPrivate::destroy()
 {
     Q_Q(QQuickWidget);
@@ -318,16 +331,8 @@ void QQuickWidgetPrivate::destroy()
 void QQuickWidgetPrivate::execute()
 {
     Q_Q(QQuickWidget);
-    ensureEngine();
+    executeHelper();
 
-    if (root) {
-        delete root;
-        root = nullptr;
-    }
-    if (component) {
-        delete component;
-        component = nullptr;
-    }
     if (!source.isEmpty()) {
         component = new QQmlComponent(engine.data(), source, q);
         if (!component->isLoading()) {
@@ -339,6 +344,20 @@ void QQuickWidgetPrivate::execute()
     }
 }
 
+void QQuickWidgetPrivate::execute(QAnyStringView uri, QAnyStringView typeName)
+{
+    Q_Q(QQuickWidget);
+    executeHelper();
+
+    component = new QQmlComponent(engine.data(), uri, typeName, q);
+    if (!component->isLoading()) {
+        q->continueExecute();
+    } else {
+        QObject::connect(component, SIGNAL(statusChanged(QQmlComponent::Status)),
+                         q, SLOT(continueExecute()));
+    }
+}
+
 void QQuickWidgetPrivate::itemGeometryChanged(QQuickItem *resizeItem, QQuickGeometryChange change,
                                               const QRectF &oldGeometry)
 {
@@ -461,7 +480,7 @@ QImage QQuickWidgetPrivate::grabFramebuffer()
                                 readResult.pixelSize.width(), readResult.pixelSize.height(),
                                 QImage::Format_RGBA8888_Premultiplied);
             if (rhi->isYUpInFramebuffer())
-                return wrapperImage.mirrored();
+                return wrapperImage.flipped();
             else
                 return wrapperImage.copy();
         }
@@ -650,6 +669,19 @@ QQuickWidget::QQuickWidget(const QUrl &source, QWidget *parent)
     setSource(source);
 }
 
+/*!
+  \since 6.9
+  Constructs a QQuickWidget with the element specified by \a uri and \a typeName
+  and parent \a parent.
+  The default value of \a parent is \c{nullptr}.
+  \sa loadFromModule
+ */
+QQuickWidget::QQuickWidget(QAnyStringView uri, QAnyStringView typeName, QWidget *parent)
+    : QQuickWidget(parent)
+{
+    loadFromModule(uri, typeName);
+}
+
 /*!
   Constructs a QQuickWidget with the given QML \a engine as a child of \a parent.
 
@@ -736,6 +768,42 @@ void QQuickWidget::setContent(const QUrl& url, QQmlComponent *component, QObject
     d->setRootObject(item);
     emit statusChanged(status());
 }
+/*!
+   Sets the initial properties \a initialProperties with which the QML
+   component gets initialized after calling \l QQuickWidget::setSource().
+
+   \note You can only use this function to initialize top-level properties.
+   \note This function should always be called before setSource, as it has
+   no effect once the component has become \c Ready.
+
+   \sa QQmlComponent::createWithInitialProperties()
+   \since 6.9
+*/
+void QQuickWidget::setInitialProperties(const QVariantMap &initialProperties)
+{
+    Q_D(QQuickWidget);
+    d->initialProperties = initialProperties;
+}
+
+/*!
+    \since 6.9
+    Loads the QML component identified by \a uri and \a typeName. If the component
+    is backed by a QML file, \l{source} will be set accordingly. For types defined
+    in \c{C++}, \c{source} will be empty.
+
+    If any \l{source} was set before this method was called, it will be cleared.
+
+    Calling this method multiple times with the same \a uri and \a typeName will result
+    in the QML component being reinstantiated.
+
+    \sa setSource, QQmlComponent::loadFromModule, QQmlApplicationEngine::loadFromModule
+ */
+void QQuickWidget::loadFromModule(QAnyStringView uri, QAnyStringView typeName)
+{
+    Q_D(QQuickWidget);
+    d->source = {}; // clear URL
+    d->execute(uri, typeName);
+}
 
 /*!
   Returns the source URL, if set.
@@ -917,6 +985,7 @@ void QQuickWidgetPrivate::updateSize()
             } else if (offscreenWindow->size().isEmpty()) {
                 // QQuickDeliveryAgentPrivate::deliverHoverEvent() ignores events that
                 // occur outside of QQuickRootItem's geometry, so we need it to match root's size.
+                offscreenWindow->resize(newSize);
                 offscreenWindow->contentItem()->setSize(newSize);
             }
         }
@@ -1105,6 +1174,7 @@ void QQuickWidget::createFramebufferObject()
     // Note: The position will be update when we get a move event (see: updatePosition()).
     const QPoint &globalPos = mapToGlobal(QPoint(0, 0));
     d->offscreenWindow->setGeometry(globalPos.x(), globalPos.y(), width(), height());
+    d->offscreenWindow->contentItem()->setSize(QSizeF(width(), height()));
 
     if (d->useSoftwareRenderer) {
         const QSize imageSize = size() * devicePixelRatio();
@@ -1125,7 +1195,17 @@ void QQuickWidget::createFramebufferObject()
     else
         samples = 0;
 
-    const QSize fboSize = size() * devicePixelRatio();
+    const int minTexSize = d->rhi->resourceLimit(QRhi::TextureSizeMin);
+    const int maxTexSize = d->rhi->resourceLimit(QRhi::TextureSizeMax);
+
+    QSize fboSize = size() * devicePixelRatio();
+    if (fboSize.width() > maxTexSize || fboSize.height() > maxTexSize) {
+        qWarning("QQuickWidget: Requested backing texture size is %dx%d, but the maximum texture size for the 3D API implementation is %dx%d",
+                 fboSize.width(), fboSize.height(),
+                 maxTexSize, maxTexSize);
+    }
+    fboSize.setWidth(qMin(maxTexSize, qMax(minTexSize, fboSize.width())));
+    fboSize.setHeight(qMin(maxTexSize, qMax(minTexSize, fboSize.height())));
 
     // Could be a simple hide - show, in which case the previous texture is just fine.
     if (!d->outputTexture) {
@@ -1240,7 +1320,9 @@ void QQuickWidget::continueExecute()
         return;
     }
 
-    QObject *obj = d->component->create();
+    std::unique_ptr<QObject> obj(d->initialProperties.empty()
+                                     ? d->component->create()
+                                     : d->component->createWithInitialProperties(d->initialProperties));
 
     if (d->component->isError()) {
         const QList<QQmlError> errorList = d->component->errors();
@@ -1252,7 +1334,13 @@ void QQuickWidget::continueExecute()
         return;
     }
 
-    d->setRootObject(obj);
+    // If we used loadFromModule, we might not have a URL so far.
+    // Thus, query the component to retrieve the associated URL, if any
+    if (d->source.isEmpty())
+        d->source = d->component->url();
+
+    if (d->setRootObject(obj.get()))
+        Q_UNUSED(obj.release());
     emit statusChanged(status());
 }
 
@@ -1260,11 +1348,12 @@ void QQuickWidget::continueExecute()
 /*!
   \internal
 */
-void QQuickWidgetPrivate::setRootObject(QObject *obj)
+bool QQuickWidgetPrivate::setRootObject(QObject *obj)
 {
     Q_Q(QQuickWidget);
     if (root == obj)
-        return;
+        return true;
+
     if (QQuickItem *sgItem = qobject_cast<QQuickItem *>(obj)) {
         root = sgItem;
         sgItem->setParentItem(offscreenWindow->contentItem());
@@ -1288,7 +1377,10 @@ void QQuickWidgetPrivate::setRootObject(QObject *obj)
             q->resize(initialSize);
         }
         initResize();
+        return true;
     }
+
+    return false;
 }
 
 QPlatformBackingStoreRhiConfig QQuickWidgetPrivate::rhiConfig() const
@@ -1313,12 +1405,6 @@ QPlatformBackingStoreRhiConfig QQuickWidgetPrivate::rhiConfig() const
 
 QWidgetPrivate::TextureData QQuickWidgetPrivate::texture() const
 {
-    Q_Q(const QQuickWidget);
-    if (!q->isWindow() && q->internalWinId()) {
-        qWarning() << "QQuickWidget cannot be used as a native child widget."
-                   << "Consider setting Qt::AA_DontCreateNativeWidgetSiblings";
-        return {};
-    }
     return { outputTexture, nullptr };
 }
 
@@ -1446,15 +1532,28 @@ void QQuickWidget::resizeEvent(QResizeEvent *e)
 bool QQuickWidget::focusNextPrevChild(bool next)
 {
     Q_D(QQuickWidget);
-    QKeyEvent event(QEvent::KeyPress, next ? Qt::Key_Tab : Qt::Key_Backtab, Qt::NoModifier);
-    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Key, QQuickProfiler::InputKeyPress, event.key(),
-                          Qt::NoModifier);
+
+    const auto *da = QQuickWindowPrivate::get(d->offscreenWindow)->deliveryAgentPrivate();
+    Q_ASSERT(da);
+
+    auto *currentTarget = da->focusTargetItem();
+    Q_ASSERT(currentTarget);
+
+    auto *nextTarget = QQuickItemPrivate::nextPrevItemInTabFocusChain(currentTarget, next, false);
+    // If no child to focus, behaves like its base class (QWidget)
+    if (!nextTarget)
+        return QWidget::focusNextPrevChild(next);
+
+    // Otherwise, simulates focus event for the offscreen window (QQuickWindow)
+    const Qt::Key k = next ? Qt::Key_Tab : Qt::Key_Backtab;
+    QKeyEvent event(QEvent::KeyPress, k, Qt::NoModifier);
+    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Key, QQuickProfiler::InputKeyPress, k, Qt::NoModifier);
     QCoreApplication::sendEvent(d->offscreenWindow, &event);
 
-    QKeyEvent releaseEvent(QEvent::KeyRelease, next ? Qt::Key_Tab : Qt::Key_Backtab, Qt::NoModifier);
-    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Key, QQuickProfiler::InputKeyRelease, releaseEvent.key(),
-                          Qt::NoModifier);
+    QKeyEvent releaseEvent(QEvent::KeyRelease, k, Qt::NoModifier);
+    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Key, QQuickProfiler::InputKeyRelease, k, Qt::NoModifier);
     QCoreApplication::sendEvent(d->offscreenWindow, &releaseEvent);
+
     return event.isAccepted();
 }
 
@@ -1611,6 +1710,24 @@ void QQuickWidget::wheelEvent(QWheelEvent *e)
 void QQuickWidget::focusInEvent(QFocusEvent * event)
 {
     Q_D(QQuickWidget);
+
+    using FocusTarget = QWindowPrivate::FocusTarget;
+    const Qt::FocusReason reason = event->reason();
+
+    switch (reason) {
+    // if there has been an item focused:
+    // set the first item focused, when the reason is TabFocusReason
+    // set the last item focused, when the reason is BacktabFocusReason
+    case Qt::TabFocusReason:
+    case Qt::BacktabFocusReason: {
+        const bool forward = reason == Qt::FocusReason::TabFocusReason;
+        const FocusTarget target = forward ? FocusTarget::First : FocusTarget::Last;
+        QQuickWindowPrivate::get(d->offscreenWindow)->setFocusToTarget(target, reason);
+    } break;
+    default:
+        break;
+    }
+
     d->offscreenWindow->focusInEvent(event);
 }
 
